import {Component} from './components';

class Tree extends Component {

  /**
   * Return configuration defaults
   * @param void
   * @return {Object}
   */
  defaults () {
    return {
      tree: {},
      cache: new Map(),
      tokenize: (expr) => expr.split('/'),
    };
  }

  /**
   * @param {Object} options
   * @param {Object} options.tree - tree data
   * @param {Object} options.cache - object that implements get(), set() and has()
   * @param {Function} options.tokenize - path expresssion tokenizer
   */
  constructor (options={}) {
    super(options);
    this.tree = this.option('tree', {pull: true});
  }

  /**
   * Tokenize the given expression on '.' and traverse the translation tree
   * until a match is found. If no match is found, return an empty string.
   * This is the main
   * @param {String} expr
   * @return {mixed}
   */
  path (expr) {
    let {cache, tokenize} = this.options;

    if (cache.has(expr)) {
      return cache.get(expr);
    }

    let traverse = (tree, token) => {
      if (tree && tree.hasOwnProperty(token)) {
        return tree[token];
      }
      throw new Error(`failed to resolve path expression "${expr}"`);
    };

    let value = tokenize(expr).reduce(traverse, this.tree);

    cache.set(expr, value);

    return value;
  }

}

export default {Tree};
