import query from '../../src/query';

describe('query', () => {
  describe('.stringify()', () => {

    it('serializes an Object', () => {
      let {object: input} = fixture('iterables-1');
      let expected = 'foo=one&bar=2';
      let actual = query.stringify(input);

      expect(actual).to.equal(expected);
    });

    it('serializes a Map', () => {
      let {map: input} = fixture('iterables-1');
      let expected = 'foo=one&bar=2';
      let actual = query.stringify(input);

      expect(actual).to.equal(expected);
    })

    it('serializes entries', () => {
      let {entries: input} = fixture('iterables-1');
      let expected = 'foo=one&bar=2';
      let actual = query.stringify(input);

      expect(actual).to.equal(expected);
    });

  });

  describe('.parse.entries()', () => {
    it('converts a query string to entries', () => {
      let input = 'foo=1&bar=two&message=Hello%20world';
      let output = query.parse.entries(input);

      expect(Array.isArray(output)).to.be.ok;
      expect(output.length).to.equal(3);

      expect(Array.isArray(output[0])).to.be.ok;
      expect(Array.isArray(output[1])).to.be.ok;
      expect(Array.isArray(output[2])).to.be.ok;

      expect(output[0][0]).to.equal('foo');
      expect(output[0][1]).to.equal('1');
      expect(output[1][0]).to.equal('bar');
      expect(output[1][1]).to.equal('two');
      expect(output[2][0]).to.equal('message');
      expect(output[2][1]).to.equal('Hello world');
    });
  });

  describe('.parse.object()', () => {
    it('converts a query string to an Object', () => {
      let input = 'foo=1&bar=two&message=Hello%20world';
      let output = query.parse.object(input);

      expect(output).to.be.an('object');
      expect(Object.keys(output).length).to.equal(3);
      expect(output.foo).to.equal('1');
      expect(output.bar).to.equal('two');
      expect(output.message).to.equal('Hello world');
    });
  });

  describe('.parse.map()', () => {
    it('converts a query string to a Map', () => {
      let input = 'foo=1&bar=two&message=Hello%20world';
      let output = query.parse.map(input);

      expect(output).to.be.an.instanceof(Map);
      expect(output.size).to.equal(3);
      expect(output.get('foo')).to.equal('1');
      expect(output.get('bar')).to.equal('two');
      expect(output.get('message')).to.equal('Hello world');
    });
  });

});
